#! /usr/local/bin/ruby -Ke

require "player"
require "gameserver"
require "message"
require "database"

class Server

  include Message
  
  START             =  0
  UNDERGROUND_START =  1
  UNDERGROUND_END   = 10
  VOLCANO_START     = 11
  VOLCANO_END       = 13
  
  def initialize

    # @gameservers[  0 ]: Ͼ塣
    # @gameservers[  1 ]: ϲµ1
    # @gameservers[ 10 ]: ϲµ10ϲµܤϤޤǡ
    # @gameservers[ 11 ]: лµ1
    # @gameservers[ 13 ]: лµ3лµܤϤޤǡ

    @gameservers = []

    @mesg     = read_mesg( $MESGFILE )
    @database = Database.new
    @request  = [] 
    
  end

  def new_game( client, hardgame )

    name = client.name
    
    client.player = ( player = Player.new( name ) )

    $HARDGAME = hardgame
    
    @database.start( name, $HARDGAME )     # Logging
    
    player.initial_equipment if ( $HARDGAME == 0 ) # 
    player.mesg = @mesg
    player.charge( @database.charge_amount( name ) ) # Ƕ⡣
    
    opening( client )
    move( client, START )
    
  end
  
  def load( client )

    name = client.name
    
    $HARDGAME = @database.hardgame( name )

    @database.load( name ).each_with_index do | cave, i |
      next unless cave
      gserver = Gameserver.new( @request, @database, i, cave )
      @gameservers[ i ] = gserver
      gserver.load( client ) unless client.player
    end

    exit unless client.player
   
  end

  # ॹȲ̤ɽ롣
  # ɤ٤˥Ϥ٤롣
   
  def accept( client )

    client.start_screen
    load_check = @database.to_load?( client.name )
    
    case ( num = client.configuration( load_check ) )
    when ?\e
      return
    when ?0 .. ?9
      new_game( client, num - ?0 )
    when ?l
      if load_check
	load( client )
      else
	return
      end
    else
      return
    end
      
    main( client )
   
  end

  def save( client )
    data = @gameservers.collect {  | gserver | gserver.cavelevel }
    @database.save( client.name, data, $HARDGAME )
    exit
  end

  def enter( client )
    create_destination( UNDERGROUND_START )
    @gameservers[ UNDERGROUND_START ].enter( client )
  end

 
  def from_underground( client )

    create_destination( START )
    @gameservers[ START ].from_underground( client )
    
  end
  
  def climb_down( client )
    
    create_destination( VOLCANO_START )
    @gameservers[ VOLCANO_START ].climb_down( client )
      
  end

  def climb_up( client )

    create_destination( START )
    @gameservers[ START ].climb_up( client )
        
  end
  
  # ŪȤʤ볬¸ߤʤȤˤϡγ롣
  def create_destination( dest )

    unless @gameservers[ dest ]
      @gameservers[ dest ] = Gameserver.new( @request, @database, dest )
    end
    
  end
  
  def move( client, dest, point = nil )

    create_destination( dest )
    @gameservers[ dest ].join( client, point ) 
    
  end

  def fall_join( client, dest, point )

    create_destination( dest )
    @gameservers[ dest ].fall_join( client, point )
    
  end

  def pit_join( client, dest, point )

    create_destination( dest )
    @gameservers[ dest ].pit_join( client, point )
    
  end
  

  def teleport_join( client, dest )

    create_destination( dest )
    @gameservers[ dest ].teleport_join( client )
    
  end

  # Gameserver顢ServerؤΥꥯ
  
  # command = [ :enter, client ]
  # clientϲµ1ء

  # command = [ :move, client, dest, point ]
  # ʤȤä̾ΰưdestء

  # command = [ :fall_join, client, dest, point ]
  # Ȥơdestء
  
  # command = [ :teleport_join, client, dest ]
  # ƥݡȤˤꡢdestء
  
  # command = [ :climb_down, client ]
  # clientлµ1( 11 )Ρ̤ζ᤯ˡ
  
  def main( client )

    loop do

      gameserver = @gameservers.find { | gserver |
	gserver && gserver.loadable?( client )
      }
      
      raise unless gameserver # ཪλ
      
      gameserver.main( client )

      while ( command = @request.shift )
	self.send( *command ) if command
      end
      
    end

  end
  
  def opening( client )
    mesg = client.player.mesg
    client.opening( mesg[ 270 .. 292 ], mesg[ 295 .. 297 ] )
  end
  
end
